# ALMA Data Reduction Script
# Tested in CASA version 5.1.1

# Calibration

thesteps = []
step_title = {0: 'Import of the ASDM',
              1: 'Fix of SYSCAL table times',
              2: 'listobs',
              3: 'A priori flagging',
              4: 'Generation and time averaging of the WVR cal table',
              5: 'Generation of the Tsys cal table',
              6: 'Generation of the antenna position cal table',
              7: 'Application of the WVR, Tsys and antpos cal tables',
              8: 'Split out science SPWs and time average',
              9: 'Listobs, clear pointing table, and save original flags',
              10: 'Initial flagging',
              11: 'Putting a model for the flux calibrator(s)',
              12: 'Save flags before bandpass cal',
              13: 'Bandpass calibration',
              14: 'Save flags before gain cal',
              15: 'Gain calibration',
              16: 'Save flags before applycal',
              17: 'Application of the bandpass and gain cal tables',
              18: 'Split out corrected column',
              19: 'Save flags after applycal'}

if 'applyonly' not in globals(): applyonly = False
try:
  print 'List of steps to be executed ...', mysteps
  thesteps = mysteps
except:
  print 'global variable mysteps not set.'
if (thesteps==[]):
  thesteps = range(0,len(step_title))
  print 'Executing all steps: ', thesteps

# The Python variable 'mysteps' will control which steps
# are executed when you start the script using
#   execfile('scriptForCalibration.py')
# e.g. setting
#   mysteps = [2,3,4]# before starting the script will make the script execute
# only steps 2, 3, and 4
# Setting mysteps = [] will make it execute all steps.

import re

import os

if applyonly != True: es = aU.stuffForScienceDataReduction() 


if re.search('^5.1.1', casadef.casa_version) == None:
 sys.exit('ERROR: PLEASE USE THE SAME VERSION OF CASA THAT YOU USED FOR GENERATING THE SCRIPT: 5.1.1')


# CALIBRATE_AMPLI: Titan
# CALIBRATE_ATMOSPHERE: J1229+0203,M100,Titan
# CALIBRATE_BANDPASS: J1229+0203
# CALIBRATE_FLUX: 
# CALIBRATE_FOCUS: 
# CALIBRATE_PHASE: J1215+1654
# CALIBRATE_POINTING: J1215+1654,J1229+0203,J1445-1629
# OBSERVE_TARGET: M100

# Using reference antenna = CM04

# Import of the ASDM
mystep = 0
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  if os.path.exists('uid___A002_X5e971a_X124.ms') == False:
    importasdm('uid___A002_X5e971a_X124', asis='Antenna Station Receiver Source CalAtmosphere CalWVR CorrelatorMode SBSummary', bdfflags=False, lazy=False)
    os.system(os.environ['CASAPATH'].split()[0]+'/bin/bdflags2MS -f "COR DELA INT MIS SIG SYN TFB WVR ZER" uid___A002_X5e971a_X124 uid___A002_X5e971a_X124.ms')
  if applyonly != True: es.fixForCSV2555('uid___A002_X5e971a_X124.ms')

# Fix of SYSCAL table times
mystep = 1
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  from recipes.almahelpers import fixsyscaltimes
  fixsyscaltimes(vis = 'uid___A002_X5e971a_X124.ms')

print "# A priori calibration"

# listobs
mystep = 2
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  os.system('rm -rf uid___A002_X5e971a_X124.ms.listobs')
  listobs(vis = 'uid___A002_X5e971a_X124.ms',
    listfile = 'uid___A002_X5e971a_X124.ms.listobs')
  
  

# A priori flagging
mystep = 3
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  flagdata(vis = 'uid___A002_X5e971a_X124.ms',
    mode = 'manual',
    spw = '0~23',
    autocorr = True,
    flagbackup = False)
  
  flagdata(vis = 'uid___A002_X5e971a_X124.ms',
    mode = 'manual',
    intent = '*POINTING*,*SIDEBAND_RATIO*,*ATMOSPHERE*',
    flagbackup = False)
  
  flagcmd(vis = 'uid___A002_X5e971a_X124.ms',
    inpmode = 'table',
    useapplied = True,
    action = 'plot',
    plotfile = 'uid___A002_X5e971a_X124.ms.flagcmd.png')
  
  flagcmd(vis = 'uid___A002_X5e971a_X124.ms',
    inpmode = 'table',
    useapplied = True,
    action = 'apply')
  
  # flag Titan since we're not going to use it
  flagdata(vis = 'uid___A002_X5e971a_X124.ms',
     mode = 'manual',
     field = 'Titan',
     flagbackup = False)



# Generation and time averaging of the WVR cal table
mystep = 4
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  

# Generation of the Tsys cal table
mystep = 5
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  os.system('rm -rf uid___A002_X5e971a_X124.ms.tsys') 
  gencal(vis = 'uid___A002_X5e971a_X124.ms',
    caltable = 'uid___A002_X5e971a_X124.ms.tsys',
    caltype = 'tsys')
  
  # Flagging edge channels
  
  flagdata(vis = 'uid___A002_X5e971a_X124.ms.tsys',
    mode = 'manual',
    spw = '8:0~3;124~127,10:0~3;124~127,12:0~3;124~127,14:0~3;124~127',
    flagbackup = False)

  if applyonly != True: aU.plotbandpass(caltable='uid___A002_X5e971a_X124.ms.tsys', overlay='time',
    xaxis='freq', yaxis='amp', subplot=22, buildpdf=False, interactive=False,
    showatm=True,pwv='auto',chanrange='92.1875%',showfdm=True, showBasebandNumber=True,
    field='', figfile='uid___A002_X5e971a_X124.ms.tsys.plots.overlayTime.flagged/uid___A002_X5e971a_X124.ms.tsys')

  flagdata(vis = 'uid___A002_X5e971a_X124.ms.tsys',
    mode = 'manual',
    field = 'Titan',
    flagbackup = True)

  # Flag bad Tsys channels for J1229 and M100 (ignore Titan since we're not going to use it)
  flagdata(vis = 'uid___A002_X5e971a_X124.ms.tsys',
    mode = 'manual',
    antenna = 'CM09',
    field='M100,J1229*',
    spw = '10:43~95',
    correlation='XX',
    flagbackup = True)

  flagdata(vis = 'uid___A002_X5e971a_X124.ms.tsys',
    mode = 'manual',
    antenna = 'CM02',
    field='M100,J1229*',
    spw = '10:32~88',
    correlation='YY',
    flagbackup = True)

  flagdata(vis = 'uid___A002_X5e971a_X124.ms.tsys',
    mode = 'manual',
    antenna = 'CM04,CM05',
    field='M100',
    spw = '10:31~88',
    correlation='XX',
    flagbackup = True)

  flagdata(vis = 'uid___A002_X5e971a_X124.ms.tsys',
    mode = 'manual',
    antenna = 'CM07,CM12',
    field='M100',
    spw = '10:36~89',
    flagbackup = True)
 
  flagdata(vis = 'uid___A002_X5e971a_X124.ms.tsys',
    mode = 'manual',
    antenna = 'CM03',
    field='M100',
    spw = '10',
    correlation='YY',
    timerange='04:40:00~05:03:22',
    flagbackup = True)

  flagdata(vis = 'uid___A002_X5e971a_X124.ms.tsys',
    mode = 'manual',
    antenna = 'CM02',
    field='M100,J1229*',
    spw = '12:48~66',
    correlation='YY',
    flagbackup = True)

  flagdata(vis = 'uid___A002_X5e971a_X124.ms.tsys',
    mode = 'manual',
    antenna = 'CM03',
    field='M100',
    spw = '12:38~78',
    correlation='XX',
    flagbackup = True)

  flagdata(vis = 'uid___A002_X5e971a_X124.ms.tsys',
    mode = 'manual',
    antenna = 'CM04',
    field='J1229*',
    spw = '12:22~85',
    correlation='XX',
    flagbackup = True)

  flagdata(vis = 'uid___A002_X5e971a_X124.ms.tsys',
    mode = 'manual',
    antenna = 'CM04',
    field='M100',
    spw = '12:33~83',
    flagbackup = True)

  flagdata(vis = 'uid___A002_X5e971a_X124.ms.tsys',
    mode = 'manual',
    antenna = 'CM07',
    field='M100,J1229*',
    spw = '12:26~80',
    flagbackup = True)

  flagdata(vis = 'uid___A002_X5e971a_X124.ms.tsys',
    mode = 'manual',
    antenna = 'CM12',
    field='M100,J1229*',
    spw = '12:40~92',
    flagbackup = True)

  flagdata(vis = 'uid___A002_X5e971a_X124.ms.tsys',
    mode = 'manual',
    antenna = 'CM05',
    field='M100',
    spw = '12:37~89',
    correlation='XX',
    flagbackup = True)

  flagdata(vis = 'uid___A002_X5e971a_X124.ms.tsys',
    mode = 'manual',
    antenna = 'CM05',
    field='M100',
    spw = '12:46~68',
    correlation='YY',
    flagbackup = True)

  flagdata(vis = 'uid___A002_X5e971a_X124.ms.tsys',
    mode = 'manual',
    antenna = 'CM06',
    field='M100',
    spw = '12:39~84',
    flagbackup = True)
 
  if applyonly != True: aU.plotbandpass(caltable='uid___A002_X5e971a_X124.ms.tsys', overlay='time',
    xaxis='freq', yaxis='amp', subplot=22, buildpdf=False, interactive=False,
    showatm=True,pwv='auto',chanrange='92.1875%',showfdm=True, showBasebandNumber=True,
    field='', figfile='uid___A002_X5e971a_X124.ms.tsys.plots.overlayTime.flagged/uid___A002_X5e971a_X124.ms.tsys')
   
  if applyonly != True: es.checkCalTable('uid___A002_X5e971a_X124.ms.tsys', msName='uid___A002_X5e971a_X124.ms', interactive=False) 
  

# Generation of the antenna position cal table
mystep = 6
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  # Note: no baseline run found for antenna CM01.
  
  # Note: no baseline run found for antenna CM04.
  
  # Position for antenna CM05 is derived from baseline run made on 2012-11-24 06:18:47.
  
  # Note: no baseline run found for antenna CM06.
  
  # Note: no baseline run found for antenna CM07.
  
  # Note: no baseline run found for antenna CM09.
  
  os.system('rm -rf uid___A002_X5e971a_X124.ms.antpos') 
  gencal(vis = 'uid___A002_X5e971a_X124.ms',
    caltable = 'uid___A002_X5e971a_X124.ms.antpos',
    caltype = 'antpos',
    antenna = 'CM04,CM06,CM07,CM09',
    parameter = [0.001386,-0.00066,-0.004261,
                    -0.000116,0.002124,-0.005089,
                    -0.003654,0.018646,0.005672,
                    0.008047,0.040179,0.00147])
  

# Application of the WVR, Tsys and antpos cal tables
mystep = 7
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  
  
  from recipes.almahelpers import tsysspwmap
  tsysmap = tsysspwmap(vis = 'uid___A002_X5e971a_X124.ms', tsystable = 'uid___A002_X5e971a_X124.ms.tsys', tsysChanTol = 1)
  
  
  
  applycal(vis = 'uid___A002_X5e971a_X124.ms',
    field = '0',
    spw = '16,18,20,22',
    gaintable = ['uid___A002_X5e971a_X124.ms.tsys', 'uid___A002_X5e971a_X124.ms.antpos'],
    gainfield = ['0', ''],
    interp = 'linear,linear',
    spwmap = [tsysmap,[]],
    calwt = True,
    flagbackup = False)
  
  
  
  # Note: J1445-1629 didn't have any Tsys measurement, and I couldn't find any close measurement. But this is not a science target, so this is probably Ok.

  # Note: J1215+1654 didn't have any Tsys measurement, so I used the one made on M100. This is probably Ok.
  
  applycal(vis = 'uid___A002_X5e971a_X124.ms',
    field = '3',
    spw = '16,18,20,22',
    gaintable = ['uid___A002_X5e971a_X124.ms.tsys', 'uid___A002_X5e971a_X124.ms.antpos'],
    gainfield = ['4', ''],
    interp = 'linear,linear',
    spwmap = [tsysmap,[]],
    calwt = True,
    flagbackup = False)
  
  
  
  applycal(vis = 'uid___A002_X5e971a_X124.ms',
    field = '4~27',
    spw = '16,18,20,22',
    gaintable = ['uid___A002_X5e971a_X124.ms.tsys', 'uid___A002_X5e971a_X124.ms.antpos'],
    gainfield = ['4', ''],
    interp = 'linear,linear',
    spwmap = [tsysmap,[]],
    calwt = True,
    flagbackup = False)
  
  if applyonly != True: es.getCalWeightStats('uid___A002_X5e971a_X124.ms') 
  

# Split out science SPWs and time average
mystep = 8
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  os.system('rm -rf uid___A002_X5e971a_X124.ms.split')
  os.system('rm -rf uid___A002_X5e971a_X124.ms.split.flagversions') 
  split(vis = 'uid___A002_X5e971a_X124.ms',
    outputvis = 'uid___A002_X5e971a_X124.ms.split',
    datacolumn = 'corrected',
    spw = '16,18,20,22',
    keepflags = True)
  
  

print "# Calibration"

# Listobs, clear pointing table, and save original flags
mystep = 9
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  os.system('rm -rf uid___A002_X5e971a_X124.ms.split.listobs')
  listobs(vis = 'uid___A002_X5e971a_X124.ms.split',
    listfile = 'uid___A002_X5e971a_X124.ms.split.listobs')
  
  tb.open('uid___A002_X5e971a_X124.ms.split/POINTING', nomodify = False)
  a = tb.rownumbers()
  tb.removerows(a)
  tb.close()
  
  if not os.path.exists('uid___A002_X5e971a_X124.ms.split.flagversions/Original.flags'):
    flagmanager(vis = 'uid___A002_X5e971a_X124.ms.split',
      mode = 'save',
      versionname = 'Original')
  
  

# Initial flagging
mystep = 10
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  # Flagging shadowed data
  
  flagdata(vis = 'uid___A002_X5e971a_X124.ms.split',
    mode = 'shadow',
    flagbackup = False)
  
  # Flagging atmospheric line(s)
  
  flagdata(vis = 'uid___A002_X5e971a_X124.ms.split',
    mode = 'manual',
    spw = '3:2548~4043',
    field = '2',
    flagbackup = False)

  # edge channels
  flagdata(vis = 'uid___A002_X5e971a_X124.ms.split',
    mode = 'manual',
    antenna = '', spw = '*:0~127,*:3952~4079',
    flagbackup = False)
  
  # Problem with CM01 (High Tsys) - not locked
  flagdata(vis = 'uid___A002_X5e971a_X124.ms.split',
    mode = 'manual',
    antenna = 'CM01',
    flagbackup = False)

  # CM07 low gain but ok

  flagdata(vis = 'uid___A002_X5e971a_X124.ms.split',
    mode = 'manual',
    spw = '0:400~470,1:128~500,2:128~200',
    flagbackup = False)

  flagdata(vis = 'uid___A002_X5e971a_X124.ms.split',
    mode = 'manual',
    spw = '2',
    antenna = 'CM09',
    correlation = 'XX',
    flagbackup = False)


# Putting a model for the flux calibrator(s)
mystep = 11
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  # from au.getALMAFluxForMS(vis)
  setjy('uid___A002_X5e971a_X124.ms.split',
        standard='manual', field='J1229+0203', spix=-0.808786,
        reffreq='107.801112GHz', fluxdensity=[7.649494,0,0,0])


# Save flags before bandpass cal
mystep = 12
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  
  flagmanager(vis = 'uid___A002_X5e971a_X124.ms.split',
    mode = 'save',
    versionname = 'BeforeBandpassCalibration')
  
  

# Bandpass calibration
mystep = 13
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  os.system('rm -rf uid___A002_X5e971a_X124.ms.split.ap_pre_bandpass') 
  
  gaincal(vis = 'uid___A002_X5e971a_X124.ms.split',
    caltable = 'uid___A002_X5e971a_X124.ms.split.ap_pre_bandpass',
    field = '0', # J1229+0203
    spw = '0:1632~2448,1:1632~2448,2:1632~2448,3:1632~2448',
    solint = 'int',
    refant = 'CM04',
    minblperant = 2,
    calmode = 'ap')
  
  if applyonly != True: es.checkCalTable('uid___A002_X5e971a_X124.ms.split.ap_pre_bandpass', msName='uid___A002_X5e971a_X124.ms.split', interactive=False) 
  
  os.system('rm -rf uid___A002_X5e971a_X124.ms.split.bandpass') 
  bandpass(vis = 'uid___A002_X5e971a_X124.ms.split',
    caltable = 'uid___A002_X5e971a_X124.ms.split.bandpass',
    field = '0', # J1229+0203
    solint = 'inf',
    combine = 'scan',
    refant = 'CM04',
    solnorm = True,
    bandtype = 'B',
    minblperant = 2,
    gaintable = 'uid___A002_X5e971a_X124.ms.split.ap_pre_bandpass')
  
  os.system('rm -rf uid___A002_X5e971a_X124.ms.split.bandpass_smooth20ch') 
  
  bandpass(vis = 'uid___A002_X5e971a_X124.ms.split',
    caltable = 'uid___A002_X5e971a_X124.ms.split.bandpass_smooth20ch',
    field = '0', # J1229+0203
    solint = 'inf,20ch',
    combine = 'scan',
    refant = 'CM04',
    solnorm = True,
    bandtype = 'B',
    minblperant = 2,
    gaintable = 'uid___A002_X5e971a_X124.ms.split.ap_pre_bandpass')
  
  if applyonly != True: es.checkCalTable('uid___A002_X5e971a_X124.ms.split.bandpass_smooth20ch', msName='uid___A002_X5e971a_X124.ms.split', interactive=False) 
  
  if applyonly != True: es.checkCalTable('uid___A002_X5e971a_X124.ms.split.bandpass', msName='uid___A002_X5e971a_X124.ms.split', interactive=False) 
  

# Save flags before gain cal
mystep = 14
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  
  flagmanager(vis = 'uid___A002_X5e971a_X124.ms.split',
    mode = 'save',
    versionname = 'BeforeGainCalibration')
  
  

# Gain calibration
mystep = 15
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  os.system('rm -rf uid___A002_X5e971a_X124.ms.split.phase_int') 
  
  gaincal(vis = 'uid___A002_X5e971a_X124.ms.split',
    caltable = 'uid___A002_X5e971a_X124.ms.split.phase_int',
    field = '0,3', # J1229+0203,J1215+1654
    solint = 'int',
    refant = 'CM04',
    gaintype = 'G',
    calmode = 'p',
    minblperant = 2,
    gaintable = 'uid___A002_X5e971a_X124.ms.split.bandpass_smooth20ch')
  
  if applyonly != True: es.checkCalTable('uid___A002_X5e971a_X124.ms.split.phase_int', msName='uid___A002_X5e971a_X124.ms.split', interactive=False) 
  
  os.system('rm -rf uid___A002_X5e971a_X124.ms.split.ampli_inf') 
  gaincal(vis = 'uid___A002_X5e971a_X124.ms.split',
    caltable = 'uid___A002_X5e971a_X124.ms.split.ampli_inf',
    field = '0,3', # J1229+0203,J1215+1654
    solint = 'inf',
    refant = 'CM04',
    gaintype = 'T',
    calmode = 'ap',
    gaintable = ['uid___A002_X5e971a_X124.ms.split.bandpass_smooth20ch', 'uid___A002_X5e971a_X124.ms.split.phase_int'])
  
  if applyonly != True: es.checkCalTable('uid___A002_X5e971a_X124.ms.split.ampli_inf', msName='uid___A002_X5e971a_X124.ms.split', interactive=False) 
  
  os.system('rm -rf uid___A002_X5e971a_X124.ms.split.flux_inf') 
  os.system('rm -rf uid___A002_X5e971a_X124.ms.split.fluxscale') 
  mylogfile = casalog.logfile()
  casalog.setlogfile('uid___A002_X5e971a_X124.ms.split.fluxscale')
  
  fluxscaleDict = fluxscale(vis = 'uid___A002_X5e971a_X124.ms.split',
    caltable = 'uid___A002_X5e971a_X124.ms.split.ampli_inf',
    fluxtable = 'uid___A002_X5e971a_X124.ms.split.flux_inf',
    reference = '0',
    transfer = '3') # 1215+1654  

  casalog.setlogfile(mylogfile)
  
  if applyonly != True: es.fluxscale2(caltable = 'uid___A002_X5e971a_X124.ms.split.ampli_inf', removeOutliers=True, msName='uid___A002_X5e971a_X124.ms', writeToFile=True, preavg=10000)
  
  os.system('rm -rf uid___A002_X5e971a_X124.ms.split.phase_inf') 
  gaincal(vis = 'uid___A002_X5e971a_X124.ms.split',
    caltable = 'uid___A002_X5e971a_X124.ms.split.phase_inf',
    field = '0,3', # J1229+0203,J1215+1654
    solint = 'inf',
    refant = 'CM04',
    gaintype = 'G',
    calmode = 'p',
    minblperant = 2,
    gaintable = 'uid___A002_X5e971a_X124.ms.split.bandpass_smooth20ch')
  
  if applyonly != True: es.checkCalTable('uid___A002_X5e971a_X124.ms.split.phase_inf', msName='uid___A002_X5e971a_X124.ms.split', interactive=False) 
  

# Save flags before applycal
mystep = 16
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  
  flagmanager(vis = 'uid___A002_X5e971a_X124.ms.split',
    mode = 'save',
    versionname = 'BeforeApplycal')
  
  

# Application of the bandpass and gain cal tables
mystep = 17
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  for i in ['0']: # J1229+0203
    applycal(vis = 'uid___A002_X5e971a_X124.ms.split',
      field = str(i),
      gaintable = ['uid___A002_X5e971a_X124.ms.split.bandpass_smooth20ch', 'uid___A002_X5e971a_X124.ms.split.phase_int', 'uid___A002_X5e971a_X124.ms.split.flux_inf'],
      gainfield = ['', i, i],
      interp = 'linear,linear',
      calwt = True,
      flagbackup = False)
  
  applycal(vis = 'uid___A002_X5e971a_X124.ms.split',
    field = '3,5~27', # M100
    gaintable = ['uid___A002_X5e971a_X124.ms.split.bandpass_smooth20ch', 'uid___A002_X5e971a_X124.ms.split.phase_inf', 'uid___A002_X5e971a_X124.ms.split.flux_inf'],
    gainfield = ['', '3', '3'], # J1215+1654
    interp = 'linear,linear',
    calwt = True,
    flagbackup = False)
  

# Split out corrected column
mystep = 18
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  os.system('rm -rf uid___A002_X5e971a_X124.ms.split.cal')
  os.system('rm -rf uid___A002_X5e971a_X124.ms.split.cal.flagversions') 
  split(vis = 'uid___A002_X5e971a_X124.ms.split',
    outputvis = 'uid___A002_X5e971a_X124.ms.split.cal',
    datacolumn = 'corrected',
    keepflags = True)
  
  

# Save flags after applycal
mystep = 19
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  
  flagmanager(vis = 'uid___A002_X5e971a_X124.ms.split.cal',
    mode = 'save',
    versionname = 'AfterApplycal')
