# ALMA Data Reduction Script
# Tested in CASA version 5.1.1

# Calibration

thesteps = []
step_title = {0: 'Import of the ASDM',
              1: 'Fix of SYSCAL table times',
              2: 'listobs',
              3: 'A priori flagging',
              4: 'Generation and time averaging of the WVR cal table',
              5: 'Generation of the Tsys cal table',
              6: 'Generation of the antenna position cal table',
              7: 'Application of the WVR, Tsys and antpos cal tables',
              8: 'Split out science SPWs and time average',
              9: 'Listobs, clear pointing table, and save original flags',
              10: 'Initial flagging',
              11: 'Putting a model for the flux calibrator(s)',
              12: 'Save flags before bandpass cal',
              13: 'Bandpass calibration',
              14: 'Save flags before gain cal',
              15: 'Gain calibration',
              16: 'Save flags before applycal',
              17: 'Application of the bandpass and gain cal tables',
              18: 'Split out corrected column',
              19: 'Save flags after applycal'}

if 'applyonly' not in globals(): applyonly = False
try:
  print 'List of steps to be executed ...', mysteps
  thesteps = mysteps
except:
  print 'global variable mysteps not set.'
if (thesteps==[]):
  thesteps = range(0,len(step_title))
  print 'Executing all steps: ', thesteps

# The Python variable 'mysteps' will control which steps
# are executed when you start the script using
#   execfile('scriptForCalibration.py')
# e.g. setting
#   mysteps = [2,3,4]# before starting the script will make the script execute
# only steps 2, 3, and 4
# Setting mysteps = [] will make it execute all steps.

import re

import os

if applyonly != True: es = aU.stuffForScienceDataReduction() 


if re.search('^5.1.1', casadef.casa_version) == None:
 sys.exit('ERROR: PLEASE USE THE SAME VERSION OF CASA THAT YOU USED FOR GENERATING THE SCRIPT: 5.1.1')


# CALIBRATE_AMPLI: 
# CALIBRATE_ATMOSPHERE: J1229+0203,M100
# CALIBRATE_BANDPASS: J1229+0203
# CALIBRATE_FLUX: 
# CALIBRATE_FOCUS: 
# CALIBRATE_PHASE: J1215+1654
# CALIBRATE_POINTING: J1215+1654,J1229+0203
# OBSERVE_TARGET: M100

# Using reference antenna = CM04

# Import of the ASDM
mystep = 0
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  if os.path.exists('uid___A002_X62f759_X4eb.ms') == False:
    importasdm('uid___A002_X62f759_X4eb', asis='Antenna Station Receiver Source CalAtmosphere CalWVR CorrelatorMode SBSummary', bdfflags=False, lazy=False)
    os.system(os.environ['CASAPATH'].split()[0]+'/bin/bdflags2MS -f "COR DELA INT MIS SIG SYN TFB WVR ZER" uid___A002_X62f759_X4eb uid___A002_X62f759_X4eb.ms')
  if applyonly != True: es.fixForCSV2555('uid___A002_X62f759_X4eb.ms')

# Fix of SYSCAL table times
mystep = 1
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  from recipes.almahelpers import fixsyscaltimes
  fixsyscaltimes(vis = 'uid___A002_X62f759_X4eb.ms')

print "# A priori calibration"

# listobs
mystep = 2
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  os.system('rm -rf uid___A002_X62f759_X4eb.ms.listobs')
  listobs(vis = 'uid___A002_X62f759_X4eb.ms',
    listfile = 'uid___A002_X62f759_X4eb.ms.listobs')
  
  

# A priori flagging
mystep = 3
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  flagdata(vis = 'uid___A002_X62f759_X4eb.ms',
    mode = 'manual',
    spw = '0~23',
    autocorr = True,
    flagbackup = False)
  
  flagdata(vis = 'uid___A002_X62f759_X4eb.ms',
    mode = 'manual',
    intent = '*POINTING*,*SIDEBAND_RATIO*,*ATMOSPHERE*',
    flagbackup = False)
  
  flagcmd(vis = 'uid___A002_X62f759_X4eb.ms',
    inpmode = 'table',
    useapplied = True,
    action = 'plot',
    plotfile = 'uid___A002_X62f759_X4eb.ms.flagcmd.png')
  
  flagcmd(vis = 'uid___A002_X62f759_X4eb.ms',
    inpmode = 'table',
    useapplied = True,
    action = 'apply')
  

# Generation and time averaging of the WVR cal table
mystep = 4
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  

# Generation of the Tsys cal table
mystep = 5
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  os.system('rm -rf uid___A002_X62f759_X4eb.ms.tsys') 
  gencal(vis = 'uid___A002_X62f759_X4eb.ms',
    caltable = 'uid___A002_X62f759_X4eb.ms.tsys',
    caltype = 'tsys')
  
  # Flagging edge channels
  
  flagdata(vis = 'uid___A002_X62f759_X4eb.ms.tsys',
    mode = 'manual',
    spw = '8:0~3;124~127,10:0~3;124~127,12:0~3;124~127,14:0~3;124~127',
    flagbackup = False)

  if applyonly != True: aU.plotbandpass(caltable='uid___A002_X62f759_X4eb.ms.tsys', overlay='time', 
    xaxis='freq', yaxis='amp', subplot=22, buildpdf=False, interactive=False,
    showatm=True,pwv='auto',chanrange='92.1875%',showfdm=True, showBasebandNumber=True, 
    field='', figfile='uid___A002_X62f759_X4eb.ms.tsys.plots.overlayTime/uid___A002_X62f759_X4eb.ms.tsys')

  flagdata(vis = 'uid___A002_X62f759_X4eb.ms.tsys',
    mode = 'manual',
    antenna = 'CM02', spw = '10:94~97',
    flagbackup = True)

  flagdata(vis = 'uid___A002_X62f759_X4eb.ms.tsys',
    mode = 'manual',
    antenna = 'CM02', spw = '12:93~97',
    flagbackup = True)

  flagdata(vis = 'uid___A002_X62f759_X4eb.ms.tsys',
    mode = 'manual',
    antenna = 'CM06', spw = '10:96~97',
    flagbackup = True)

  flagdata(vis = 'uid___A002_X62f759_X4eb.ms.tsys',
    mode = 'manual',
    antenna = 'CM06', spw = '12:95~97',
    flagbackup = True)

  flagdata(vis = 'uid___A002_X62f759_X4eb.ms.tsys',
    mode = 'manual',
    antenna = 'CM07', spw = '10:99~101',
    flagbackup = True)

  flagdata(vis = 'uid___A002_X62f759_X4eb.ms.tsys',
    mode = 'manual',
    antenna = 'CM07', spw = '12:99~101',
    flagbackup = True)

  if applyonly != True: aU.plotbandpass(caltable='uid___A002_X62f759_X4eb.ms.tsys', overlay='time', 
    xaxis='freq', yaxis='amp', subplot=22, buildpdf=False, interactive=False,
    showatm=True,pwv='auto',chanrange='92.1875%',showfdm=True, showBasebandNumber=True, 
    field='', figfile='uid___A002_X62f759_X4eb.ms.tsys.plots.overlayTime/uid___A002_X62f759_X4eb.ms.tsys') 
  
  
  if applyonly != True: es.checkCalTable('uid___A002_X62f759_X4eb.ms.tsys', msName='uid___A002_X62f759_X4eb.ms', interactive=False) 
  

# Generation of the antenna position cal table
mystep = 6
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  # Position for antenna CM09 is derived from baseline run made on 2013-05-10 03:34:01.
  
  os.system('rm -rf uid___A002_X62f759_X4eb.ms.antpos') 
  gencal(vis = 'uid___A002_X62f759_X4eb.ms',
    caltable = 'uid___A002_X62f759_X4eb.ms.antpos',
    caltype = 'antpos',
    antenna = 'CM09',
    parameter = [5.46308586478e-05,0.000510964064225,7.80525102297e-05])
  

# Application of the WVR, Tsys and antpos cal tables
mystep = 7
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  
  
  from recipes.almahelpers import tsysspwmap
  tsysmap = tsysspwmap(vis = 'uid___A002_X62f759_X4eb.ms', tsystable = 'uid___A002_X62f759_X4eb.ms.tsys', tsysChanTol = 1)
  
  
  
  applycal(vis = 'uid___A002_X62f759_X4eb.ms',
    field = '0',
    spw = '16,18,20,22',
    gaintable = ['uid___A002_X62f759_X4eb.ms.tsys', 'uid___A002_X62f759_X4eb.ms.antpos'],
    gainfield = ['0', ''],
    interp = 'linear,linear',
    spwmap = [tsysmap,[]],
    calwt = True,
    flagbackup = False)
  
  
  
  # Note: J1215+1654 didn't have any Tsys measurement, so I used the one made on M100. This is probably Ok.
  
  applycal(vis = 'uid___A002_X62f759_X4eb.ms',
    field = '1',
    spw = '16,18,20,22',
    gaintable = ['uid___A002_X62f759_X4eb.ms.tsys', 'uid___A002_X62f759_X4eb.ms.antpos'],
    gainfield = ['2', ''],
    interp = 'linear,linear',
    spwmap = [tsysmap,[]],
    calwt = True,
    flagbackup = False)
  
  
  
  applycal(vis = 'uid___A002_X62f759_X4eb.ms',
    field = '2~25',
    spw = '16,18,20,22',
    gaintable = ['uid___A002_X62f759_X4eb.ms.tsys', 'uid___A002_X62f759_X4eb.ms.antpos'],
    gainfield = ['2', ''],
    interp = 'linear,linear',
    spwmap = [tsysmap,[]],
    calwt = True,
    flagbackup = False)
  
  if applyonly != True: es.getCalWeightStats('uid___A002_X62f759_X4eb.ms') 
  

# Split out science SPWs and time average
mystep = 8
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  os.system('rm -rf uid___A002_X62f759_X4eb.ms.split') 
  os.system('rm -rf uid___A002_X62f759_X4eb.ms.split.flagversions') 
  split(vis = 'uid___A002_X62f759_X4eb.ms',
    outputvis = 'uid___A002_X62f759_X4eb.ms.split',
    datacolumn = 'corrected',
    spw = '16,18,20,22',
    keepflags = True)
  
  

print "# Calibration"

# Listobs, clear pointing table, and save original flags
mystep = 9
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  os.system('rm -rf uid___A002_X62f759_X4eb.ms.split.listobs')
  listobs(vis = 'uid___A002_X62f759_X4eb.ms.split',
    listfile = 'uid___A002_X62f759_X4eb.ms.split.listobs')
  
  tb.open('uid___A002_X62f759_X4eb.ms.split/POINTING', nomodify = False)
  a = tb.rownumbers()
  tb.removerows(a)
  tb.close()
  
  if not os.path.exists('uid___A002_X62f759_X4eb.ms.split.flagversions/Original.flags'):
    flagmanager(vis = 'uid___A002_X62f759_X4eb.ms.split',
      mode = 'save',
      versionname = 'Original')
  
  

# Initial flagging
mystep = 10
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  # Flagging shadowed data
  
  flagdata(vis = 'uid___A002_X62f759_X4eb.ms.split',
    mode = 'shadow',
    flagbackup = False)

  # Flagging edge channels and bad data

  flagdata(vis = 'uid___A002_X62f759_X4eb.ms.split',
    mode = 'manual',
    antenna = '', spw = '*:0~127,*:3952~4079',
    flagbackup = False)

  flagdata(vis = 'uid___A002_X62f759_X4eb.ms.split',
    mode = 'manual',
    antenna = '', spw = '1:3000~3280,2:2990~3150,2:3225~3280',
    flagbackup = False)

  flagdata(vis = 'uid___A002_X62f759_X4eb.ms.split',
    mode = 'manual',
    antenna = 'CM07', spw = '2:3151~3250',
    flagbackup = False)

  flagdata(vis = 'uid___A002_X62f759_X4eb.ms.split',
    mode = 'manual',
    antenna = '', spw = '0:400~470',
    flagbackup = False)
  
#  flagdata(vis = 'uid___A002_X62f759_X4eb.ms.split',
#    mode = 'manual',
#    antenna = 'CM02,CM06,CM08',
#    spw = '1:3000~3080',
#    flagbackup=False)
 

# Putting a model for the flux calibrator(s)
mystep = 11
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  # set flux scale by hand
  setjy(vis = 'uid___A002_X62f759_X4eb.ms.split',
    field = 'J1229+0203', # field 0
    spw = '0,1',
    fluxdensity = [7.8,0,0,0],
    standard = 'manual',
    usescratch = False)

  setjy(vis = 'uid___A002_X62f759_X4eb.ms.split',
    field = 'J1229+0203', # field 0
    spw = '2,3',
    fluxdensity = [7.4,0,0,0],
    standard = 'manual',
    usescratch = False)
  

# Save flags before bandpass cal
mystep = 12
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  
  flagmanager(vis = 'uid___A002_X62f759_X4eb.ms.split',
    mode = 'save',
    versionname = 'BeforeBandpassCalibration')
  
  

# Bandpass calibration
mystep = 13
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  os.system('rm -rf uid___A002_X62f759_X4eb.ms.split.ap_pre_bandpass') 

  gaincal(vis = 'uid___A002_X62f759_X4eb.ms.split',
    caltable = 'uid___A002_X62f759_X4eb.ms.split.ap_pre_bandpass',
    field = '0', # J1229+0203
    spw = '0:1632~2448,1:1632~2448,2:1632~2448,3:1632~2448',
    solint = 'int',
    refant = 'CM04',
    minblperant = 2,
    calmode = 'ap')
  
  if applyonly != True: es.checkCalTable('uid___A002_X62f759_X4eb.ms.split.ap_pre_bandpass', msName='uid___A002_X62f759_X4eb.ms.split', interactive=False) 
  
  os.system('rm -rf uid___A002_X62f759_X4eb.ms.split.bandpass') 
  bandpass(vis = 'uid___A002_X62f759_X4eb.ms.split',
    caltable = 'uid___A002_X62f759_X4eb.ms.split.bandpass',
    field = '0', # J1229+0203
    solint = 'inf',
    combine = 'scan',
    refant = 'CM04',
    solnorm = True,
    bandtype = 'B', 
    minblperant = 2,
    gaintable = 'uid___A002_X62f759_X4eb.ms.split.ap_pre_bandpass')
  
  os.system('rm -rf uid___A002_X62f759_X4eb.ms.split.bandpass_smooth20ch') 
  bandpass(vis = 'uid___A002_X62f759_X4eb.ms.split',
    caltable = 'uid___A002_X62f759_X4eb.ms.split.bandpass_smooth20ch',
    field = '0', # J1229+0203
    solint = 'inf,20ch',
    combine = 'scan',
    refant = 'CM04',
    solnorm = True,
    bandtype = 'B',
    minblperant = 2,
    gaintable = 'uid___A002_X62f759_X4eb.ms.split.ap_pre_bandpass')
  
  if applyonly != True: es.checkCalTable('uid___A002_X62f759_X4eb.ms.split.bandpass_smooth20ch', msName='uid___A002_X62f759_X4eb.ms.split', interactive=False) 
  
  if applyonly != True: es.checkCalTable('uid___A002_X62f759_X4eb.ms.split.bandpass', msName='uid___A002_X62f759_X4eb.ms.split', interactive=False) 
  

# Save flags before gain cal
mystep = 14
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  
  flagmanager(vis = 'uid___A002_X62f759_X4eb.ms.split',
    mode = 'save',
    versionname = 'BeforeGainCalibration')
  
  

# Gain calibration
mystep = 15
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  os.system('rm -rf uid___A002_X62f759_X4eb.ms.split.phase_int') 
  gaincal(vis = 'uid___A002_X62f759_X4eb.ms.split',
    caltable = 'uid___A002_X62f759_X4eb.ms.split.phase_int',
    field = '0~1', # J1229+0203,J1215+1654
    solint = 'int',
    refant = 'CM04',
    gaintype = 'G',
    calmode = 'p',
    minblperant = 2,
    gaintable = 'uid___A002_X62f759_X4eb.ms.split.bandpass_smooth20ch')
  
  if applyonly != True: es.checkCalTable('uid___A002_X62f759_X4eb.ms.split.phase_int', msName='uid___A002_X62f759_X4eb.ms.split', interactive=False) 

  os.system('rm -rf uid___A002_X62f759_X4eb.ms.split.ampli_inf')
  gaincal(vis = 'uid___A002_X62f759_X4eb.ms.split',
    caltable = 'uid___A002_X62f759_X4eb.ms.split.ampli_inf',
    field = 'J1229+0203,J1215+1654',
    solint = 'inf',
    refant = 'CM04',
    gaintype = 'T',
    calmode = 'ap',
    minblperant = 2,
    gaintable = ['uid___A002_X62f759_X4eb.ms.split.bandpass_smooth20ch', 'uid___A002_X62f759_X4eb.ms.split.phase_int'])

  if applyonly != True: es.checkCalTable('uid___A002_X62f759_X4eb.ms.split.ampli_inf', msName='uid___A002_X62f759_X4eb.ms.split', interactive=False)

  os.system('rm -rf uid___A002_X62f759_X4eb.ms.split.flux_inf') 
  os.system('rm -rf uid___A002_X62f759_X4eb.ms.split.fluxscale') 
  mylogfile = casalog.logfile()
  casalog.setlogfile('uid___A002_X62f759_X4eb.ms.split.fluxscale')

  fluxscale(vis = 'uid___A002_X62f759_X4eb.ms.split',
    caltable = 'uid___A002_X62f759_X4eb.ms.split.ampli_inf',
    fluxtable = 'uid___A002_X62f759_X4eb.ms.split.flux_inf',
    reference = 'J1229+0203',
    transfer = 'J1215+1654')

  casalog.setlogfile(mylogfile) 

  os.system('rm -rf uid___A002_X62f759_X4eb.ms.split.phase_inf') 
  gaincal(vis = 'uid___A002_X62f759_X4eb.ms.split',
    caltable = 'uid___A002_X62f759_X4eb.ms.split.phase_inf',
    field = '0~1', # J1229+0203,J1215+1654
    solint = 'inf',
    refant = 'CM04',
    gaintype = 'G',
    calmode = 'p',
    minblperant = 2,
    gaintable = 'uid___A002_X62f759_X4eb.ms.split.bandpass_smooth20ch')
  
  if applyonly != True: es.checkCalTable('uid___A002_X62f759_X4eb.ms.split.phase_inf', msName='uid___A002_X62f759_X4eb.ms.split', interactive=False) 
  

# Save flags before applycal
mystep = 16
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  
  flagmanager(vis = 'uid___A002_X62f759_X4eb.ms.split',
    mode = 'save',
    versionname = 'BeforeApplycal')
  
  

# Application of the bandpass and gain cal tables
mystep = 17
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  for i in ['0']: # J1229+0203
    applycal(vis = 'uid___A002_X62f759_X4eb.ms.split',
      field = str(i),
      gaintable = ['uid___A002_X62f759_X4eb.ms.split.bandpass_smooth20ch', 'uid___A002_X62f759_X4eb.ms.split.phase_int', 'uid___A002_X62f759_X4eb.ms.split.flux_inf'],
      gainfield = ['', i, i],
      interp = 'linear,linear',
      calwt = True,
      flagbackup = False)
  
  applycal(vis = 'uid___A002_X62f759_X4eb.ms.split',
    field = '1,3~25', # M100
    gaintable = ['uid___A002_X62f759_X4eb.ms.split.bandpass_smooth20ch', 'uid___A002_X62f759_X4eb.ms.split.phase_inf', 'uid___A002_X62f759_X4eb.ms.split.flux_inf'],
    gainfield = ['', '1', '1'], # J1215+1654
    interp = 'linear,linear',
    calwt = True,
    flagbackup = False)
  

# Split out corrected column
mystep = 18
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  os.system('rm -rf uid___A002_X62f759_X4eb.ms.split.cal')
  os.system('rm -rf uid___A002_X62f759_X4eb.ms.split.cal.flagversions') 
  split(vis = 'uid___A002_X62f759_X4eb.ms.split',
    outputvis = 'uid___A002_X62f759_X4eb.ms.split.cal',
    datacolumn = 'corrected',
    keepflags = True)
  


# Save flags after applycal
mystep = 19
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  
  flagmanager(vis = 'uid___A002_X62f759_X4eb.ms.split.cal',
    mode = 'save',
    versionname = 'AfterApplycal')

